// Copyright 2016 Tadd Torborg -- I grant the right to modify and sell products using this sourcecode.  
//I insist you put my copyright notice and this line in every sourcecode file included in any project which uses my PWRMAN software. 
// If you do manage to construct or produce 2 or more devices based on this source code, please send an email to me at tadd@mac.com
//  Thanks!! 
// Source code by Tadd Torborg, ham-radio callsign KA2DEW, for PWRMAN power controller -- runs on TI MSP430F2012 CPU.
// Concept by Tadd Torborg.  Board design, layout, stuffed, and maintained by Stephen Snell, ham-radio callsign W4RFQ
// PWRMAN operation is documented on the TARPN web site.  See  http://tarpn.net/pwrman
// I compiled and debugged this project using IAR's EW430 version 5.
//
/// IAR or TI Library includes
#include "string.h"
#include "stdlib.h"
#include "io430.h"


//PWRMAN includes
#include "common.h"
#include "sw_led.h"
#include "pihellotoggle.h"


///###########
/// PI HELLO TOGGLE is a GPIO output from the Raspberry PI to the PWRMAN.
/// Once the Raspberry PI discovers that the loopback through the ribbon
/// cable is connected, it starts generating a square wave to the PWRMAN
/// which signals that the Raspberry PI knows about the PWRMAN and is OK.
/// If the PWRMAN does not see the toggle, the PWRMAN will cease driving
/// the Raspberry PI GPIOs.
///
/// This module contains the code to determine of the Raspberry PI is
/// alive.  The output of this module is a variable: GoodTransitionsInARow_G.
/// If the PI is not detected, that output will be 0.  The measure of
/// success is a constant: REQUIRED_PI_HELLO_GOOD_TRANSITIONS_FOR_CONFIDENCE
///
/// This module has two functions, Init() and Update().   Init is called at
/// boot time.  Update is called from mainloop once per main-loop.  The
/// assumption is that mainloop will occur at least once per half-cycle
/// of the Raspberry PI's square wave.  The expectation is that the
/// square wave has a 1hz cycle or slower and mainloop is quite a bit
/// faster than that.   The timeout for a change int he HelloToggle input
/// in the MSP430 CPU is PI_HELLO_HALFCYCLE_TIMEOUT_INIT which was 30 as
/// of version 2.
///
///
/// A note on programming style:
/// Functions are capitalized and have CamelCase
/// Variables are not capitalized and have Camlecase.
/// All Variables end with _l _L or _G.
/// _l is local to a function
/// _L is global within a single source code module
/// _G is global to the project.  Only _G variables will show in include files.
///
/// All globals are extern in include files.
///
/// None of the project's include files will include any other file.
///
/// Functions may change _L and _G variables (yes, that sucks, but it saved stack space and CPU space).
///


unsigned int  piHelloToggle_GoodTransitionsInARow_G;  //Increments every time we see the input change.  max of REQUIRED___ .  Set to 0 if timeout.
unsigned int  piHelloToggle_ChangeTimeoutCounter_L;   //Decrements each time Update is called until it reaches zero at which point we think the Raspberry PI is gone.
unsigned char piHelloToggle_Status_L;                 //This has the prior (last Update call) value of the rPI Hello Toggle signal.
unsigned int  notAtZero_L;                            //>0 if we were not timed out the last time Update() was called.


//#############################################
// INIT PI-HELLO TOGGLE
//
// Called at RESET to to first initialization of the variables used to detect the PI-HELLO cycle.
void InitPiHelloToggle(void)
{
   piHelloToggle_ChangeTimeoutCounter_L  = PI_HELLO_HALFCYCLE_TIMEOUT_INIT;
   piHelloToggle_Status_L                = 0;
   piHelloToggle_GoodTransitionsInARow_G = 0;
   notAtZero_L                           = FALSE;
}



//#############################################
// UPDATE PI HELLO-TO-US SENSE
//
// This function keeps track of whether the Raspberry is connected to us and awake.
// If it is awake we are supposed to get a 1 cycle per second square wave from the
// Raspberry PI.
//
// Called from mainloop cycle once per cycle
void UpdatePiHelloToggle(void)
{

   if(piHelloToggle_ChangeTimeoutCounter_L > 0)
   {
      piHelloToggle_ChangeTimeoutCounter_L--;
      notAtZero_L = TRUE;
   }
   if(piHelloToggle_ChangeTimeoutCounter_L == 0)     //The Raspberry PI is not answering.  Either not connected or powered down.
   {
      if(notAtZero_L == TRUE)                        //if we though it was up before, go into this fragmenet
      {
         notAtZero_L = FALSE;                        //we thought it was up.  Well it isn't up now.
         piHelloToggle_GoodTransitionsInARow_G = 0;  //no soup for you!  We've lost touch with the Raspberry PI.  PI had better be powered off!
         piHelloToggle_ChangeTimeoutCounter_L  = PI_HELLO_HALFCYCLE_TIMEOUT_INIT;
      }
   }

   {
      unsigned char piBlinkyInput_l;
      piBlinkyInput_l = ReadPiHelloToggle();           //Get the .5hz toggle signal from the Raspberry PI
      if(piHelloToggle_Status_L != piBlinkyInput_l)    //Check if it has changed.                                                       //It did change.  Count another good transition in a row.
      {
         piHelloToggle_ChangeTimeoutCounter_L = PI_HELLO_HALFCYCLE_TIMEOUT_INIT;
         piHelloToggle_Status_L               = piBlinkyInput_l;
         if(piHelloToggle_GoodTransitionsInARow_G < REQUIRED_PI_HELLO_GOOD_TRANSITIONS_FOR_CONFIDENCE)
         {
            piHelloToggle_GoodTransitionsInARow_G++;
         }
      }
   }
}


